import re
from dataclasses import dataclass
from typing import Optional

email_regex = re.compile(
    r"[a-z0-9!#$%&'*+/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&'*+/=?^_`{|}~-]+)*@(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?"  # noqa: E501
)


@dataclass
class Result:
    """Consists of a result after finding (or not) and email."""

    is_email: bool
    email: str
    line_number: int
    file_name: str

    def __str__(self):
        return f"{self.line_number} - {self.email} - {self.file_name}"


def is_email(line: str, line_number: int, file_name: str) -> Optional[Result]:
    """Detects if a given text is an email or not and returns a Result if so,
    None otherwise.

    :param line: text that might be an email
    :param line_number: line number where the email is at
    :param file_name: name of the file the email is at
    :return: Result object if line is email, None otherwise
    """
    if re.fullmatch(email_regex, line):
        return Result(is_email=True, email=line, line_number=line_number, file_name=file_name)
    return None



def read_lines_from_file(filename: str) -> list[str]:
    """Reads lines from a file and returns them as list of str.

    :param filename: file name to read lines from
    :return: list of str containing each line
    """
    with open(filename, "r") as file:
        return file.readlines()



import ntpath

import chinaski.read_lines as rl
from chinaski.finder import find_emails_in_line
from chinaski.is_email import Result, is_email


def path_leaf(path: str) -> str:
    """Retrieves the file name and extension from a given path.

    It should work in different OS.

    Obtained from:
    https://stackoverflow.com/questions/8384737/extract-file-name-from-path-no-matter-what-the-os-path-format

    :param path: path to retrieve the file from
    :return: str with the file name and extension
    """
    head, tail = ntpath.split(path)
    return tail or ntpath.basename(head)


class FilePathIsNotValidException(Exception):
    def __init__(self, message="The given file path is not valid"):
        super().__init__(message)


def core(file_path: str) -> list[Result]:
    """Core function of the program.

    It will read the lines from a given file and return a list of Result where it has found emails.

    :param file_path: path to file to detect emails in it
    :return: list of Result, containing the findings
    """
    if file_path == "" or file_path is None:
        raise FilePathIsNotValidException

    lines = rl.read_lines_from_file(filename=file_path)
    results = []
    file_name = path_leaf(file_path)

    for i in range(len(lines)):
        line = lines[i]
        emails_in_line = find_emails_in_line(line)

        for email in emails_in_line:
            res = is_email(line=email, line_number=i + 1, file_name=file_name)
            if res:
                results.append(res)

    return results
